/*
 * PSP Software Development Kit - http://www.pspdev.org
 * -----------------------------------------------------------------------
 * Licensed under the BSD license, see LICENSE in PSPSDK root for details.
 *
 * main.c - Basic PSPSDK sample.
 *
 * Copyright (c) 2005 Marcus R. Brown <mrbrown@ocgnet.org>
 * Copyright (c) 2005 James Forshaw <tyranid@gmail.com>
 * Copyright (c) 2005 John Kelley <ps2dev@kelley.ca>
 *
 * $Id: main.c 363 2005-06-27 20:35:14Z tyranid $
 */
#include "pg.c"
#include "background.c"
#include "rock.c"
#include "paper.c"
#include "scissors.c"
#include <pspkernel.h>
#include <pspdebug.h>
#include <stdlib.h>
#include <string.h>

/* Define the module info section */
PSP_MODULE_INFO("SDKTEST", 0, 1, 1);

/* Define the main thread's attribute value (optional) */
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER | THREAD_ATTR_VFPU);

/* Define printf, just to make typing easier */
#define printf	pspDebugScreenPrintf
#define RGB(r, g, b) ((r)|((g)<<5)|((b)<<10))
#define TEXT_COLOR RGB(0, 0, 0)

int win;
int lose;

/* Exit callback */
int exit_callback(void)
{
	sceKernelExitGame();

	return 0;
}

/* Callback thread */
void CallbackThread(void *arg)
{
	int cbid;

	cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
	sceKernelRegisterExitCallback(cbid);

	sceKernelSleepThreadCB();
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void)
{
	int thid = 0;

	thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, THREAD_ATTR_USER, 0);
	if(thid >= 0)
	{
		sceKernelStartThread(thid, 0, 0);
	}

	return thid;
}

void fillCell(int x, int y)
{
	unsigned short* vram = (unsigned short*) pgGetVramAddr(x * 16, y * 16);
	int xo, yo;
	for (yo = 0; yo < 16; yo++) {
		for (xo = 0; xo < 16; xo++) {
			vram[xo + yo * 512] = backgroundData[16*x + xo + (16*y + yo) * backgroundWidth];
		}
	}
}

void plotCellImage(int x, int y, unsigned short* image)
{
	unsigned short* vram = (unsigned short*) pgGetVramAddr(x, y);
	int xo, yo;
	for (yo = 0; yo < 64; yo++) {
		for (xo = 0; xo < 64; xo++) {
			if (*image != 0x8000) {
				vram[xo + yo * 512] = *image;
			} else vram[xo + yo * 512] = backgroundData[64*x + xo + (64*y + yo) * backgroundWidth];
			image++;
		}
	}
}

const char digitFont[] = {
	1,1,1,1,
	1,0,0,1,
	1,0,0,1,
	1,0,0,1,
	1,1,1,1,
	
	0,0,0,1,
	0,0,0,1,
	0,0,0,1,
	0,0,0,1,
	0,0,0,1,
	
	1,1,1,1,
	0,0,0,1,
	1,1,1,1,
	1,0,0,0,
	1,1,1,1,
	
	1,1,1,1,
	0,0,0,1,
	1,1,1,1,
	0,0,0,1,
	1,1,1,1,
	
	1,0,0,1,
	1,0,0,1,
	1,1,1,1,
	0,0,0,1,
	0,0,0,1,
	
	1,1,1,1,
	1,0,0,0,
	1,1,1,1,
	0,0,0,1,
	1,1,1,1,
	
	1,1,1,1,
	1,0,0,0,
	1,1,1,1,
	1,0,0,1,
	1,1,1,1,
	
	1,1,1,1,
	0,0,0,1,
	0,0,0,1,
	0,0,0,1,
	0,0,0,1,
	
	1,1,1,1,
	1,0,0,1,
	1,1,1,1,
	1,0,0,1,
	1,1,1,1,

	1,1,1,1,
	1,0,0,1,
	1,1,1,1,
	0,0,0,1,
	1,1,1,1};

void print7Segment(int x, int y, int digit)
{
	unsigned short* vram = (unsigned short*) pgGetVramAddr(x, y);
	int xo, yo;
	int i = digit * 5*4;
	for (yo = 0; yo < 5; yo++) {
		for (xo = 0; xo < 4; xo++) {
			if (digitFont[i]) {
				vram[xo + yo * 512] = TEXT_COLOR;
			} else {
				vram[xo + yo * 512] = backgroundData[x + xo + (y + yo) * backgroundWidth];
			}
			i++;
		}
	}
		
}

void printDecimal(int x, int y, int color, int value)
{
	int i;
	int digits[6];
	for (i = 5; i >= 0; i--) {
		digits[i] = value%10;
		value /= 10;
	}
	i = 0;
	while (i < 5 && digits[i] == 0) digits[i++] = -1;
	for (i = 0; i < 6; i++) {
		if (digits[i] >= 0) {
			print7Segment(x, y, digits[i]);
			x += 6;
		}
	}
}



void newGame()
{
	pgFlipDrawFrame();
	unsigned short* vram = (unsigned short*) pgGetVramAddr(0, 0);
	int xo, yo;
	unsigned short* image = backgroundData;
	for (yo = 0; yo < backgroundHeight; yo++) {
		for (xo = 0; xo < backgroundWidth; xo++) {
			if (*image != 0x8000) vram[xo + yo * 512] = *image;
			image++;
		}
	}

	printDecimal(28, 202, TEXT_COLOR, win);
	printDecimal(28, 216, TEXT_COLOR, lose);

	pgFlipShowFrameV();
}

void writeByte(int fd, unsigned char data)
{
	sceIoWrite(fd, &data, 1);
}

const char tgaHeader[] = {0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0};

void screenshot()
{
	const int width = 480;
	const int lineWidth = 512;
	const int height = 272;
	unsigned short lineBuffer[width];
	unsigned short* vram = (unsigned short*) pgGetVramAddr(0, 0);
	int x, y;
	int fd = sceIoOpen("ms0:/screenshot.tga", PSP_O_CREAT | PSP_O_TRUNC | PSP_O_WRONLY, 0777);
	if (!fd) return;
	sceIoWrite(fd, tgaHeader, sizeof(tgaHeader));
	writeByte(fd, width & 0xff);
	writeByte(fd, width >> 8);
	writeByte(fd, height & 0xff);
	writeByte(fd, height >> 8);
	writeByte(fd, 16);
	writeByte(fd, 0);
	for (y = height - 1; y >= 0; y--) {
		for (x = 0; x < width; x++) {
			unsigned short color = vram[y * lineWidth + x];
			unsigned char red = color & 0x1f;
			unsigned char green = (color >> 5) & 0x1f;
			unsigned char blue = (color >> 10) & 0x1f;
			lineBuffer[x] = blue | (green<<5) | (red<<10);
		}
		sceIoWrite(fd, lineBuffer, width * 2);
	}
	sceIoClose(fd);
}

int main(void)
{
	ctrl_data_t pad;
	int number;
	win = 0;
	lose = 0;
	//pspDebugScreenInit();
	SetupCallbacks();

	pgInit();
	pgScreenFrame(1, 1);
	
	newGame();
	printDecimal(28, 202, TEXT_COLOR, win);
	printDecimal(28, 216, TEXT_COLOR, lose);
	srand(sceKernelLibcTime((void *) 0));
	while (1) {
		sceCtrlReadBufferPositive(&pad, 1);
		number = 1 + rand() % 3;

		//Rock is circle
		if(pad.buttons & CTRL_CIRCLE)
		{
			//Your hand
			plotCellImage(239,162,rockData);
			pgWaitVn(50);
			if(number==1){ pgPrint(0,1,0xffff,"Tie!"); 	plotCellImage(175,162,rockData);}
			if(number==2){ pgPrint(0,1,0xffff,"You win psp had scissors"); win++; 	plotCellImage(175,162,scissorsData);}
			if(number==3){ pgPrint(0,1,0xffff,"You lose psp had paper :("); lose++; plotCellImage(175,162,paperData);}
			if(number == 2) printDecimal(28, 202, TEXT_COLOR, win);
			if(number == 3) printDecimal(28, 216, TEXT_COLOR, lose);
			pgWaitVn(100);
			newGame();
		}
		//Paper is triangle
		if(pad.buttons & CTRL_TRIANGLE)
		{
			//Your hand
			plotCellImage(239,162,paperData);
			pgWaitVn(50);
			if(number==1){ pgPrint(0,1,0xffff,"You win psp had rock!"); win++; plotCellImage(175,162,rockData);}
			if(number==2){ pgPrint(0,1,0xffff,"You lose psp had scissors :("); lose++; plotCellImage(175,162,scissorsData);}
			if(number==3){ pgPrint(0,1,0xffff,"Tie!"); plotCellImage(175,162,paperData);}
			if(number == 1) printDecimal(28, 202, TEXT_COLOR, win); 
			if(number == 2) printDecimal(28, 216, TEXT_COLOR, lose);
			pgWaitVn(100);
			newGame();
		}
		//Scissors is square
		if(pad.buttons & CTRL_SQUARE)
		{
			//Your hand
			plotCellImage(239,162,scissorsData);
			pgWaitVn(50);
			if(number==1){ pgPrint(0,1,0xffff,"You lose psp had rock :("); lose++;plotCellImage(175,162,rockData);}
			if(number==2){ pgPrint(0,1,0xffff,"Tie!"); plotCellImage(175,162,scissorsData);}
			if(number==3){ pgPrint(0,1,0xffff,"You win psp had paper!"); win++; plotCellImage(175,162,paperData);}
			if(number == 3) printDecimal(28, 202, TEXT_COLOR, win);
			if(number == 1) printDecimal(28, 216, TEXT_COLOR, lose);
			pgWaitVn(100);
			newGame();
		}
		//Take a screen shot
		if(pad.buttons & CTRL_CROSS)
		{
			screenshot();
			pgWaitVn(50);
		}
	}

	return 0;
}
